package components;

import javax.swing.*; 

import components.Empire.Cell;

import tools.*;
import turchin.Turchin;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
/*
 * @author rholguin
 *
 * This is the main panel that will be dispalyed to the user.  It will show the the before and after grid visual
 * it will also give them a user interface to allow them to adjust the parameters to whatever they like and
 * re-run the simulation
 */
public class MainPanel extends JPanel implements ActionListener, Runnable {
    
	private JLabel hLabel;
	private TextField hText;
	
	private JLabel rLabel; 
	private TextField rText;
	
	private JLabel gLabel; 
	private TextField gText;
	
	private JLabel criticalLabel;
	private TextField criticalText;
	
	private JLabel beforeLabel;
	private JLabel afterLabel;
	
	private JLabel refreshLabel;
	private TextField refreshText;
	
	private JLabel iterationLabel; 
	private TextField iterationText;
	
	private JButton simButton;
	private JButton quit;
	
	
	private int rValue = 40;
	private int hValue = 30;
	private int gValue = 20;
	private int criticalValue = 40;
	private double refreshValue = 0;
	private int iterationValue = 2;
	private boolean runSim = true;
	private Random ran;
	
	private MyDialog dialog;
	private JFrame frame;
	private ReadFile file;
	private WriteFile writeFile;
	private Simulation sim;
	
	private CreateGrid before;
	private CreateGrid after;
	
	private RandomGrid ranGrid;
	
	public MainPanel(JFrame f, Turchin turch) { 
		setLayout(null);
		setSize(765, 690);
		
		frame = f;
		ran = new Random();
		
		//create components to be displayed
		beforeLabel = new JLabel("Before:");
		afterLabel = new JLabel("After:");
		
		hLabel = new JLabel("h Value: 0. ");
		hText = new TextField(""+hValue,2);
		
		rLabel = new JLabel("r Value: 0.");
		rText = new TextField(""+rValue,2);
		
		gLabel = new JLabel("g Value: 0.");
		gText = new TextField(""+gValue,2);
		
		refreshLabel = new JLabel("Refresh Rate (seconds):");
		refreshText = new TextField(""+refreshValue,2);
		
		iterationLabel = new JLabel("# of iterations:");
		iterationText = new TextField(""+iterationValue,2);
		
		criticalLabel = new JLabel("Critical Value: 0.");
		criticalText = new TextField(""+criticalValue);
		
		simButton = new JButton("Run Simulation");
		simButton.addActionListener(this);
		
		quit = new JButton("Quit Simulation");
		quit.setName("quit");
		quit.addActionListener(this);
		
		file = new ReadFile();
		
		if(file.getError() != null)
			Error(file.getError());
		before = new CreateGrid(file.getGrid());
		after = new CreateGrid(file.getGrid());
		removeAll();
		
		//add components to teh panel and set thier locations
		add(criticalLabel);
		add(criticalText);
		add(afterLabel);
		add(beforeLabel);
		add(gLabel);
		add(gText);
		add(hLabel);
		add(hText);
		add(rLabel);
		add(rText);
		add(simButton);
		add(quit);
		add(before);
		add(after);
		add(refreshLabel);
		add(refreshText);
		add(iterationLabel);
		add(iterationText);
		
		rLabel.setBounds(410,50,rLabel.getPreferredSize().width,rLabel.getPreferredSize().height);
		rText.setBounds(410+rLabel.getPreferredSize().width,50,30,20);
		criticalLabel.setBounds(570,50,criticalLabel.getPreferredSize().width,criticalLabel.getPreferredSize().height);
		criticalText.setBounds(570+criticalLabel.getPreferredSize().width,50,30,20);
		before.setBounds(20,180,250,250);
		beforeLabel.setBounds(120,150,beforeLabel.getPreferredSize().width, beforeLabel.getPreferredSize().width);
		after.setBounds(430,180,250,250);
		afterLabel.setBounds(530,150,beforeLabel.getPreferredSize().width, beforeLabel.getPreferredSize().width);
		
		gLabel.setBounds(250,50,gLabel.getPreferredSize().width,gLabel.getPreferredSize().height);
		gText.setBounds(250+gLabel.getPreferredSize().width,50,30,20);
		hLabel.setBounds(70,50,hLabel.getPreferredSize().width, hLabel.getPreferredSize().height);
		hText.setBounds(70+hLabel.getPreferredSize().width,50,30,20);
		
		refreshLabel.setBounds(270,80,refreshLabel.getPreferredSize().width,refreshLabel.getPreferredSize().height);
		refreshText.setBounds(refreshLabel.getPreferredSize().width+270,80,30,20);
		iterationLabel.setBounds(300,110,iterationLabel.getPreferredSize().width,iterationLabel.getPreferredSize().height);
		iterationText.setBounds(iterationLabel.getPreferredSize().width+300,110,30,20);
		
		simButton.setBounds(300,140,simButton.getPreferredSize().width,simButton.getPreferredSize().height);
		quit.setBounds(290,410,quit.getPreferredSize().width,quit.getPreferredSize().height);
	}
	
	public void actionPerformed(ActionEvent e) {
		try {
			hValue = new Integer(hText.getText()).intValue();
			rValue = new Integer(rText.getText()).intValue();
			gValue = new Integer(gText.getText()).intValue();
			criticalValue = new Integer(criticalText.getText()).intValue();
			refreshValue = new Double(refreshText.getText()).doubleValue();
			iterationValue = new Integer(iterationText.getText()).intValue();
		
			//each if statment is mapped to a button
			if(e.getActionCommand().equals("Run Simulation")) {
			    runSim = true;
			    simButton.setEnabled(false);
				sim = new Simulation(file.getGrid(),hValue,rValue,gValue,criticalValue);  //create simulation
				sim.initSim(true);  //init sim
				writeFile = new WriteFile();  //create output file reader
				writeFile.write(sim.getEmpireData());  //write initial data
				(new Thread(this)).start();  //start simulation thread
			}
			else if(e.getActionCommand().equals("Quit Simulation")) {
				runSim = false;  //quit simulation
				simButton.setEnabled(true);
			}
		} catch (NumberFormatException n) {
		    //input was not a valid number
			dialog = new MyDialog(frame,"Invalid input, input must be an number and all fields must be filled in",false);
			dialog.setVisible(true);
		}
	}
	
	public void run() {
	    after.updateGrid(sim.getNewGrid());  //update visuals of grid
	    int temp = 0;
		try {
			Thread.sleep((int)(1000));  //refresh delay
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
	    //while there are still iterations left and user did not press quit
		while(iterationValue > 0 && runSim) {
		    
			//ArrayList cellList = sim.getCellList();  //get new cell list to iterate
			sim.resetCounters();  //reset counters
			
			//iterate through the cell list
			while(!sim.isCellListEmpty() && runSim) {
				//int ranIndex = ran.nextInt(cellList.size());  //get random cell
				Cell c = sim.getNewCell();
				sim.runSim(c);  //run simulation on that cell
				
				after.updateGrid(sim.getNewGrid());  //update visuals of grid
				try {
					Thread.sleep((int)(refreshValue*1000.0)+5);  //refresh delay
				} catch (InterruptedException e) {
					e.printStackTrace();
				}
				//cellList.remove(c);  //remove cell from the list
			}
			writeFile.write(sim.getEmpireData());  //write iteration data to file
			iterationValue--;
			sim.initSim(false);  //re-initiallize the simulation data
		}
		writeFile.done();
		simButton.setEnabled(true);
		dialog = new MyDialog(frame,"Simulation Finished",false);
		dialog.setVisible(true);
	}
	
	//create error pop-up message
	private void Error(String text) {
		dialog = new MyDialog(frame, text, true);
		dialog.setVisible(true);
	}
	
	private boolean validateInput(int d) {
		return false;
	}
}