package tools;

import components.Empire.*;

import java.util.*;
import java.util.regex.Pattern;
/**
 * @author rholguin
 *
 * TODO To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
public class Simulation {
	
	private String[] origGrid; //hold copy of the original grid
	private String[] newGrid;  //holds the new updated grid
	private Cell[][] cellGrid; //matrix of Cells on the X-Y Plane
	
	private ArrayList availableIndex;  //list of available Empire indexes
	private ArrayList empires;  //list of current empires
	private ArrayList cellList;  //single list of all cells on the cellGrid
	
	private Random ran;
	
	//user inputted value from the main page
	private int hValue;
	private int rValue;
	private int gValue;
	private int critValue;
	
	private int collapsedEmpires;  //counts collapsed empires
	private int createdEmpires;  //counts created Empires
	
	/*
	 * Initialize some of the key variables that will be used throughout the simulation
	 */
	public Simulation(String[] g, int hV, int rV, int gV, int cV) {
		origGrid = g;
		newGrid = (String[])origGrid.clone();
		ran = new Random();
		cellGrid = new Cell[origGrid.length][origGrid.length];
		empires = new ArrayList();
		availableIndex = new ArrayList();
		
		hValue = hV;
		rValue = rV;
		gValue = gV;
		critValue = cV;
	}
	
	/*
	 * This will do all the needed processes to initialize teh simulaiton and get it ready to be run
	 */
	public void initSim(boolean newSim) {
		//if this is a brand-new simulation
		if(newSim) {
			ArrayList empireIndex = new ArrayList();  //contains the index values for all the empires
		
			/*
			 * fill list with all the possible empire indexes.  The index factor is for mapping values to colors.
			 * since colors will be reused and i have 64 colors, my maximum number of indexes must be a factor
			 * of 64 so that the value will map the color map properly.  so if there is 81 possible (9x9) 
			 * then the indexFactor will be 2, which means i will make room for 128 possible indexes.  
			 * so if an index ends up being 70, I will do 70-64 = 14...so 10 will map to teh same color as 14 does.
			 */ 
			int indexFactor = (int)(Math.pow(origGrid.length,2) / 64)+1;
			for(int x = 1; x < 64*indexFactor; x++) {
				availableIndex.add(new Integer(x));
			}
		
			/*
			* This will parse the grid and look for all the different empires on the grid.
			* For every new empire it finds it will add that index to the empireIndex list
			* and then remove that index value from the availableIndex list
			*/
			for(int x = 0; x < origGrid.length; x++) {
				Pattern pattern = Pattern.compile("[\\s]*[\\s]");
				String[] temp = pattern.split(newGrid[x]);
			
				for(int y = 0; y < origGrid.length; y++) {
					Integer index = new Integer(temp[y]);
					if(!empireIndex.contains(index)) {
						empireIndex.add(index);
						availableIndex.remove(index);
					}
				}
			}
			
			initEmpires(empireIndex);
			cellList = createCellGrid();
		}
		//if only generating a new CellList
		else {
			cellList = createCellGrid();
		}
	}
	
	//return the list of empires
	public ArrayList getEmpires() {
		return empires;
	}
	
	public Cell getNewCell() {
	    Cell c = (Cell)cellList.get(ran.nextInt(cellList.size()));
	    cellList.remove(c);
	    return c;
	}
	
	public boolean isCellListEmpty() {
	    return cellList.isEmpty();
	}
	
	/*
	 * This will create take the empireIndex list and create a new empire for each element in the list
	 * It will then search the grid and find cells for that particular emipre.  When a cell is found it
	 * is then added to the empire
	 */
	private void initEmpires(ArrayList list) {
		Iterator temp = list.iterator();
		
		//iterate through all the empiresIndex
		while(temp.hasNext()) {
			int index = ((Integer)temp.next()).intValue();
			Empire empire = new Empire(index);  //create a new empire for that index
			
			for(int x = 0; x < origGrid.length; x++) {
				Pattern pattern = Pattern.compile("[\\s]*[\\s]");
				String[] tempString = pattern.split(origGrid[x]);
				
				for(int y = 0; y < origGrid.length; y++) {
					int tempIndex = new Integer(tempString[y]).intValue();
					//if a cell is found with that same empire index then it is added to the empire
					if( tempIndex == index) {
						Cell c = new Cell(index,x,y);
						empire.addCell(c);
					}
				}
			}
			empires.add(empire);  //addes new empire to the current empire list
		}
	}
	
	/*
	 * This will take the given cell and send it the the process that will determind if it will fight
	 * or not
	 */
	public void runSim(Cell c) {
		int x = c.getXCoord();
		int y = c.getYCoord();
			
		checkNeighbors(x,y,c.getIndex());
	}
	
	/*
	 * Resets all the neccessary counters
	 */
	public void resetCounters() {
		collapsedEmpires = 0;
		createdEmpires = 0;
	}
	
	/*
	 * This will check to see if there are any empires that are eligible to collapse.  An empire will
	 * collapse if the avgSolidarity of the empire is lower than the critical Value
	 */
	public void checkEmpireCollapse() {
		Iterator iterator = empires.iterator();
		ArrayList collapseEmp = new ArrayList();
		//iterate through the empires
		while(iterator.hasNext()) {
			Empire empire = (Empire)iterator.next();
			
			//check the avgSolidarty, but note that since 0 is a free cell empire it cannot collapse
			if(empire.getAvgSolidarity() < (double)critValue/100.0 && empire.getIndex() != 0) {
				availableIndex.add(new Integer(empire.getIndex()));
				collapsedEmpires++;
				collapseEmp.add(empire);
			}
		}
		
		if(!collapseEmp.isEmpty()) {
		    empireCollapse(collapseEmp);
		}
	}
	
	/*
	 * return the cell list
	 */
	public ArrayList getCellList() {
		return cellList;
	}
	
	/*
	 * This will generate an array of data that will be used when writing the output file.  It will
	 * output the number of empires, how many collapsed and how many were created during the
	 * given iteration
	 */
	public int[] getEmpireData() {
		int counter = 0;
		Iterator iterator = empires.iterator();
		
		while (iterator.hasNext()) {
			Empire empire = (Empire)iterator.next();
			if(empire.getIndex() != 0) {
				counter++;
			}
		}
		
		int[] data = {counter,collapsedEmpires, createdEmpires};
		return data;
	}
	
	/*
	 * This method is called when a new updated grid needs to be created for the visual representation
	 */
	public String[] getNewGrid() {
		String temp = "";
		
		//Cycle through the cell matrix and append the cell index to the string array
		for(int x = 0; x < cellGrid.length; x++) {
			for(int y = 0; y < cellGrid.length; y++) {
				temp += cellGrid[x][y].getIndex()+" ";
			}
			newGrid[x] = temp;
			temp = "";  //new string needs to be created for the next row
		}
		return newGrid;
	}
	
	/*
	 * The inputted empire is marked to collapse so this method will process that request
	 */
	private void empireCollapse(ArrayList empireList) {
		Iterator iterator = empires.iterator();
		Empire nonEmpire = null;
		
		//This will find the free cell empire (index 0)
		while(iterator.hasNext()) {
			Empire temp = (Empire)iterator.next();
			if(temp.getIndex() == 0) {
				nonEmpire = temp;
				break;
			}
		}
		
		Iterator empireIt = empireList.iterator();
		while(empireIt.hasNext()) {
		    Empire tempEmp = (Empire)empireIt.next();
		    Iterator cellIterator = tempEmp.getCells().iterator();
		    Cell tempCell = null;
		    
		    int count = 0;
		    
		    //iterate through the collapsed empires cells
		    while(cellIterator.hasNext()) {
		        tempCell = (Cell)cellIterator.next();
		        
		        //duplicate the cell except set its index to 0, then add it to the free empire cell ist
		        Cell newCell = new Cell(0,tempCell.getXCoord(),tempCell.getYCoord());
		        newCell.setSolidarity(tempCell.getSolidarity());
		        nonEmpire.addCell(newCell);
		        
		        cellGrid[tempCell.getXCoord()][tempCell.getYCoord()] = newCell;  //update the cellGrid
		    }
		    
		    tempEmp.Destroy();
		}
	}
	
	/*
	 * This will generate the CellGrid matrix
	 */
	private ArrayList createCellGrid() {
		ArrayList list = new ArrayList();
		
		Iterator tempEmpire = empires.iterator();
		
		//iterate through all teh emipres, get all teh cells and add them to teh temp list of cells.
		while(tempEmpire.hasNext()) {
			Empire emp = (Empire)tempEmpire.next();
			
			Iterator tempCell = emp.getCells().iterator();
			while(tempCell.hasNext()) {
				list.add((Cell)tempCell.next());
			}
		}
		
		Iterator tempList = list.iterator();
		
		//iterate through that temp list of cells and get the X-Y coordinates and set the cell
		//in the appropriate position on the grid.
		while(tempList.hasNext()) {
			Cell c = (Cell)tempList.next();
			cellGrid[c.getXCoord()][c.getYCoord()] = c;
		}
		
		setPowerLevels();  //set teh power levels for all the cells
		return list;
	}
	
	/*
	 * This will iterate through all of the empires and set the power levels for all the cells
	 * in that empire
	 */
	private void setPowerLevels() {
		Iterator tempEmpire = empires.iterator();
		while(tempEmpire.hasNext()) {
			Empire emp = (Empire)tempEmpire.next();
			emp.setCellPower(hValue);
		}
	}
	
	/*
	 * This will check to see if there are any emipres that are empty.  If they are empty then they
	 * need to be removed
	 */
	private void checkEmptyEmpires() {
		Iterator tempEmpire = empires.iterator();
		ArrayList removeEmp = new ArrayList();
		
		while(tempEmpire.hasNext()) {
			Empire emp = (Empire)tempEmpire.next();
			
			//The 0 empire should not be removed because its techically not a real empire, all free
			//state cells will go here
			if(emp.getNumCells() == 0 && emp.getIndex() != 0) {
				availableIndex.add(new Integer(emp.getIndex()));
				removeEmp.add(emp);
				//empires.remove(emp);  //remove the empire from the empire list
				//break;
			}
		}
		
		tempEmpire = removeEmp.iterator();
		while(tempEmpire.hasNext()) {
		    empires.remove((Empire)tempEmpire.next());
		}
	}
	
	/*
	 * This will take the current cell and determine which of its Moore neighbors it is eligible to fight
	 * It will only fight cells that do NOT have the same index.
	 */
	private void checkNeighbors(int x, int y, int index) {
		ArrayList fightCells = new ArrayList();  //list to hold all cells that can be fought
		if(x == 0 && y == 0) {
			if(cellGrid[x][y+1].getIndex() != index || (cellGrid[x][y+1].getIndex() == 0 && index == 0)) fightCells.add(cellGrid[x][y+1]);
			if(cellGrid[x+1][y].getIndex() != index || (cellGrid[x+1][y].getIndex() == 0 && index == 0)) fightCells.add(cellGrid[x+1][y]);
		}
		else if(x == cellGrid.length-1 && y == 0) {
			if(cellGrid[x][y+1].getIndex() != index || (cellGrid[x][y+1].getIndex() == 0 && index == 0)) fightCells.add(cellGrid[x][y+1]);
			if(cellGrid[x-1][y].getIndex() != index || (cellGrid[x-1][y].getIndex() == 0 && index == 0)) fightCells.add(cellGrid[x-1][y]);
		}
		else if(x == 0 && y == cellGrid.length-1) {
			if(cellGrid[x][y-1].getIndex() != index || (cellGrid[x][y-1].getIndex() == 0 && index == 0)) fightCells.add(cellGrid[x][y-1]);
			if(cellGrid[x+1][y].getIndex() != index || (cellGrid[x+1][y].getIndex() == 0 && index == 0)) fightCells.add(cellGrid[x+1][y]);
		}
		else if(x == cellGrid.length-1 && y == cellGrid.length-1) {
			if(cellGrid[x][y-1].getIndex() != index || (cellGrid[x][y-1].getIndex() == 0 && index == 0)) fightCells.add(cellGrid[x][y-1]);
			if(cellGrid[x-1][y].getIndex() != index || (cellGrid[x-1][y].getIndex() == 0 && index == 0)) fightCells.add(cellGrid[x-1][y]);
		}
		else if(y == 0) {
			if(cellGrid[x-1][y].getIndex() != index || (cellGrid[x-1][y].getIndex() == 0 && index == 0)) fightCells.add(cellGrid[x-1][y]);
			if(cellGrid[x][y+1].getIndex() != index || (cellGrid[x][y+1].getIndex() == 0 && index == 0)) fightCells.add(cellGrid[x][y+1]);
			if(cellGrid[x+1][y].getIndex() != index || (cellGrid[x+1][y].getIndex() == 0 && index == 0)) fightCells.add(cellGrid[x+1][y]);
		}
		else if(y == cellGrid.length-1) {
			if(cellGrid[x-1][y].getIndex() != index || (cellGrid[x-1][y].getIndex() == 0 && index == 0)) fightCells.add(cellGrid[x-1][y]);
			if(cellGrid[x][y-1].getIndex() != index || (cellGrid[x][y-1].getIndex() == 0 && index == 0)) fightCells.add(cellGrid[x][y-1]);
			if(cellGrid[x+1][y].getIndex() != index || (cellGrid[x+1][y].getIndex() == 0 && index == 0)) fightCells.add(cellGrid[x+1][y]);
		}
		else if(x == 0) {
			if(cellGrid[x][y-1].getIndex() != index || (cellGrid[x][y-1].getIndex() == 0 && index == 0)) fightCells.add(cellGrid[x][y-1]);
			if(cellGrid[x+1][y].getIndex() != index || (cellGrid[x+1][y].getIndex() == 0 && index == 0)) fightCells.add(cellGrid[x+1][y]); 
			if(cellGrid[x][y+1].getIndex() != index || (cellGrid[x][y+1].getIndex() == 0 && index == 0)) fightCells.add(cellGrid[x][y+1]); 
		}
		else if(x == cellGrid.length-1) {
			if(cellGrid[x][y-1].getIndex() != index || (cellGrid[x][y-1].getIndex() == 0 && index == 0)) fightCells.add(cellGrid[x][y-1]); 
			if(cellGrid[x-1][y].getIndex() != index || (cellGrid[x-1][y].getIndex() == 0 && index == 0)) fightCells.add(cellGrid[x-1][y]); 
			if(cellGrid[x][y+1].getIndex() != index || (cellGrid[x][y+1].getIndex() == 0 && index == 0)) fightCells.add(cellGrid[x][y+1]); 
		}
		else {
			if(cellGrid[x+1][y].getIndex() != index || (cellGrid[x+1][y].getIndex() == 0 && index == 0)) fightCells.add(cellGrid[x+1][y]); 
			if(cellGrid[x-1][y].getIndex() != index || (cellGrid[x-1][y].getIndex() == 0 && index == 0)) fightCells.add(cellGrid[x-1][y]); 
			if(cellGrid[x][y+1].getIndex() != index || (cellGrid[x][y+1].getIndex() == 0 && index == 0)) fightCells.add(cellGrid[x][y+1]); 
			if(cellGrid[x][y-1].getIndex() != index || (cellGrid[x][y-1].getIndex() == 0 && index == 0)) fightCells.add(cellGrid[x][y-1]);
		}
		
		/*
		 * if the fightCells list is not empty then that means that there were cells found
		 * that can be fought.  So a cell is choosen at random from that list and sent to the
		 * cellBattle method
		 */
		if(!fightCells.isEmpty()) {
			int ranIndex = ran.nextInt(fightCells.size());
			Cell c = (Cell)fightCells.get(ranIndex);
			cellBattle(x,y,c.getXCoord(),c.getYCoord());
		}
	}
	
	/*
	 * Two cells will battle each other and the loser will become part of the empire that the
	 * winner belongs too
	 */
	private void cellBattle(int x1, int y1, int x2, int y2) {
		Cell c1 = cellGrid[x1][y1];
		Cell c2 = cellGrid[x2][y2];
		
		//check the power levels against each other, the cell with the highest power level will
		//win the battle.
		if(c1.getPower() > c2.getPower()) {
			switchEmpires(c1,c2);
		}
		else {
			switchEmpires(c2,c1);
		}
		
		//After any cell movements/battles these conditions need to be checked again.
		
		checkEmpireCollapse();
		checkEmptyEmpires();
		setPowerLevels();
	}
	
	/*
	 * When a cell loses a battle then that cell becomes part of teh winnering cells empire
	 * This method will process that.
	 */
	private void switchEmpires(Cell winner, Cell loser) {
		Iterator iterator = empires.iterator();
		Empire winEmpire = null;
		Empire loseEmpire = null;
		
		//find the empre of the winning , find the empire of the losing cell
		while(iterator.hasNext()) {
			Empire temp = (Empire)iterator.next();
			if(temp.getIndex() == winner.getIndex()) {
				winEmpire = temp;
			}
			if(temp.getIndex() == loser.getIndex()) {
				loseEmpire = temp;
			}
		}
		
		/*
		 * If the cell is not 0, then duclipace the lossing cell with the winning cells Index
		 * Then set the new solidarity level for the new cell, add it to the winning empire
		 * and remoeve the losing cell from the losing empire
		 * 
		 * If the winning cell is a 0 empire, then a new empire will be created
		 */
		if(winner.getIndex() != 0) {
			Cell newCell = new Cell(winner.getIndex(),loser.getXCoord(),loser.getYCoord());
			newCell.setSolidarity((double)((winner.getSolidarity()+loser.getSolidarity())/2.0));
			winEmpire.addCell(newCell);
			loseEmpire.removeCell(loser);
					
			cellGrid[loser.getXCoord()][loser.getYCoord()] = newCell;
		}
		else {
			createNewEmpire(winner,winEmpire,loser,loseEmpire);
		}
	}
	
	/*
	 * This method will create a new empire based on the winning cell.
	 */
	private void createNewEmpire(Cell winner, Empire winEmpire, Cell loser, Empire loseEmpire) {
		
	    //randomly choose a index for this new empire and then remove that index from the
	    //available index list
	    int newIndex = 0;
	    newIndex = ((Integer)availableIndex.get(ran.nextInt(availableIndex.size()))).intValue();
	    
	    availableIndex.remove(new Integer(newIndex));
		
		Empire newEmpire = new Empire(newIndex);  //new empire with the new index
		
		//replicate the losing cell but change the index to the new empires index, add to new empire
		Cell newCell = new Cell(newIndex,loser.getXCoord(),loser.getYCoord());
		newCell.setSolidarity((double)((winner.getSolidarity()+loser.getSolidarity())/2.0));
		newEmpire.addCell(newCell);
		cellGrid[loser.getXCoord()][loser.getYCoord()] = newCell;
		
		//replicate the winning cell but change the index to the new empires index, add to new empire
		newCell = new Cell(newIndex,winner.getXCoord(),winner.getYCoord());
		newCell.setSolidarity(winner.getSolidarity());
		newEmpire.addCell(newCell);
		cellGrid[winner.getXCoord()][winner.getYCoord()] = newCell;
	
		//remove the cells from thier previous empires
		loseEmpire.removeCell(loser);
		winEmpire.removeCell(winner);
	
		//add new empire to the empire list
		empires.add(newEmpire);
		createdEmpires++;
	}
}